/*
 * Synopsis:
 *	int disrfst(int stream, size_t achars, char *value)
 *
 *	Gets a Data-is-Strings character string from <stream> and converts it
 *	into an ASCII NUL-terminated string, and puts the string into <value>,
 *	a pre-allocated string, <achars> long.  The character string in <stream>
 *	consists of an unsigned integer, followed by a number of characters
 *	determined by the unsigned integer.
 *
 *	Disrfst returns DIS_SUCCESS if everything works well.  It returns an
 *	error code otherwise.  In case of an error, the <stream> character
 *	pointer is reset, making it possible to retry with some other conversion
 *	strategy, and the first character of <value> is set to ASCII NUL.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <stddef.h>
#include <stdlib.h>

#ifndef NDEBUG
#include <string.h>
#endif

#include "dis.h"
#include "dis_.h"

int disrfst(stream, achars, value)
    int			stream;
    size_t		achars;
    char		*value;
    {
        int		locret;
        int		negate;
        unsigned	count;

	assert(value != NULL);
	assert(dis_gets != NULL);
	assert(disr_commit != NULL);

	locret = disrsi_(stream, &negate, &count, 1);
	if (locret == DIS_SUCCESS) {
		if (negate)
		        locret = DIS_BADSIGN;
		else if (count > achars)
		        locret = DIS_OVERFLOW;
		else if ((*dis_gets)(stream, value, (size_t)count) !=
						(size_t)count)
		        locret = DIS_PROTO;
#ifndef NDEBUG
		else if (memchr(value, 0, (size_t)count))
		        locret = DIS_NULLSTR;
#endif
		else
		        value[count] = '\0';
	}
	locret = (*disr_commit)(stream, locret == DIS_SUCCESS) ?
	    DIS_NOCOMMIT : locret;
	if (locret != DIS_SUCCESS)
	        *value = '\0';
	return (locret);
}
